# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class AppointmentProvider(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    GETLABS = "getlabs"
    AXLEHEALTH = "axlehealth"
    PHLEBFINDERS = "phlebfinders"
    QUEST = "quest"
    _UNKNOWN = "__APPOINTMENTPROVIDER_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "AppointmentProvider":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        getlabs: typing.Callable[[], T_Result],
        axlehealth: typing.Callable[[], T_Result],
        phlebfinders: typing.Callable[[], T_Result],
        quest: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is AppointmentProvider.GETLABS:
            return getlabs()
        if self is AppointmentProvider.AXLEHEALTH:
            return axlehealth()
        if self is AppointmentProvider.PHLEBFINDERS:
            return phlebfinders()
        if self is AppointmentProvider.QUEST:
            return quest()
        return _unknown_member(self._value_)
