# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.placeholder import Placeholder
from ...types.relative_timeframe import RelativeTimeframe

QueryBatchTimeframe = typing.Union[RelativeTimeframe, Placeholder]
