# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Interpretation(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    NORMAL = "normal"
    ABNORMAL = "abnormal"
    CRITICAL = "critical"
    _UNKNOWN = "__INTERPRETATION_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "Interpretation":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        normal: typing.Callable[[], T_Result],
        abnormal: typing.Callable[[], T_Result],
        critical: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is Interpretation.NORMAL:
            return normal()
        if self is Interpretation.ABNORMAL:
            return abnormal()
        if self is Interpretation.CRITICAL:
            return critical()
        return _unknown_member(self._value_)
